<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.2 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Category.php,v 1.2 2005/04/15 13:07:46 mike Exp $
 */

/**#@+
 * Categories static variables
 * @ignore
 */
$GLOBALS['_Gadget']['Category'] = array(
    'Store'     => array(),
    'Setups'    => array(),
    'Current'   => '',
    'Context'   => null,
);
/**#@-*/

/**
 * Gadget_Category
 * 
 * @package     Gadget
 * @since       1.0
 */
class Gadget_Category
{
    /**
     * Set Context
     * 
     * @static
     * @access  public
     * @return  void
     * @param   string  $context
     */
    function setContext($context)
    {
        $GLOBALS['_Gadget']['Category']['Current'] = $context;
        $GLOBALS['_Gadget']['Category']['Context'] =
            &$GLOBALS['_Gadget']['Category']['Store'][$context];
    }
    
    /**
     * Set Setup
     * 
     * @static
     * @access  public
     * @return  bool
     * @param   string  $context
     * @param   mixed   $callback
     */
    function setSetup($context, $callback)
    {
        if ($isCallable = is_callable($callback)) {
            $GLOBALS['_Gadget']['Category']['Setups'][$context] = $callback;
        }
        return $isCallable;
    }
    
    /**
     * Set DAO
     * 
     * @static
     * @access  public
     * @return  void
     * @param   string  $dao
     */
    function setDAO($dao)
    {
        $GLOBALS['_Gadget']['Category']['Setups'][$dao] = 
            array('Gadget_Category', 'setupDAO');
    }
    
    /**
     * Setup
     * 
     * @static
     * @access  protected
     * @return  void
     */
    function setup()
    {
        $context = $GLOBALS['_Gadget']['Category']['Current'];
        $GLOBALS['_Gadget']['Category']['Context'] = call_user_func(
            $GLOBALS['_Gadget']['Category']['Setups'][$context]
        );
    }
    
    /**
     * Setup through DAO
     * 
     * @static
     * @access  protected
     * @return  array
     */
    function setupDAO()
    {
        $cat = array();
        $dao = Gadget::DAO($GLOBALS['_Gadget']['Category']['Current']);
        if (!PEAR::isError($dao) && $dao->find()) {
            while ($dao->fetch()) {
                $cat[$dao->id] = $dao->toArray();
            }
        }
        return $cat;
    }
    
    /**
     * Get Category Parts
     * 
     * @static
     * @access  public
     * @return  array
     * @param   array   $parts
     * @param   array   $search
     * @param   bool    $distinct
     * @param   bool    $invert
     * @param   string  $compare
     */
    function getParts($part, $search = null, $distinct = true, $invert = false, $compare = '==')
    {
        if ($search) {
            $cats = Gadget_Category::search($search, $distinct, $invert, $compare);
        } else {
            $cats = Gadget_Category::get();
        }
        $result = array();
        foreach ($cats as $cat) {
            isset($cat[$part]) and $result[$cat['id']] = $cat[$part];
        }
        return $result;
    }
    
    /**
     * Get Category
     * 
     * @static
     * @access  public
     * @return  mixed
     * @param   mixed
     * @param   string
     */
    function get($id = null, $part = null)
    {
        if (!isset($GLOBALS['_Gadget']['Category']['Context'])) {
            Gadget_Category::setup();
        }
        
        if (isset($id, $part)) {
            return isset($GLOBALS['_Gadget']['Category']['Context'][$id][$part]) ?
                $GLOBALS['_Gadget']['Category']['Context'][$id][$part] : array();
        }
        
        if (isset($id)) {
            return isset($GLOBALS['_Gadget']['Category']['Context'][$id]) ?
                $GLOBALS['_Gadget']['Category']['Context'][$id] : array();
        }
        
        return $GLOBALS['_Gadget']['Category']['Context'];
    }
    
    /**
     * Search Category
     * 
     * @static
     * @access  public
     * @return  array
     * @param   array   $search
     * @param   bool    $distinct
     * @param   bool    $invert
     * @param   string  $compare
     */
    function search($search, $distinct = true, $invert = false, $compare = '==')
    {
        $result = array();
        
        if ($distinct) {
            if ($invert) {
                foreach (Gadget_Category::get() as $c) {
                    $add = true;
                    foreach ((array) $search as $part => $id) {
                        if (Gadget_Category::compare($compare, $c[$part], $id)) {
                            $add = false;
                        }
                    }
                    if ($add) {
                        $result[$c['id']] = $c;
                    }
                }
            } else {
                foreach (Gadget_Category::get() as $c) {
                    $add = true;
                    foreach ((array) $search as $part => $id) {
                        if (!Gadget_Category::compare($compare, $c[$part], $id)) {
                            $add = false;
                        }
                    }
                    if ($add) {
                        $result[$c['id']] = $c;
                    }
                }
            }
        } else {
            if ($invert) {
                foreach ((array) $search as $part => $id) {
                    foreach (Gadget_Category::get() as $c) {
                        if (!Gadget_Category::compare($compare, $c[$part], $id)) {
                            $result[$c['id']] = $c;
                        }
                    }
                }
            } else {
                foreach ((array) $search as $part => $id) {
                    foreach (Gadget_Category::get() as $c) {
                        if (Gadget_Category::compare($compare, $c[$part], $id)) {
                            $result[$c['id']] = $c;
                        }
                    }
                }
            }
        }
        
        return $result;
    }
    
    /**
     * Compare
     * 
     * @static
     * @access  protected
     * @return  bool
     */
    function compare($operator, &$L, &$R)
    {
        switch ($operator)
        {
            case '===': return $L === $R;
            case '==':  return $L == $R;
            case '!==': return $L !== $R;
            case '!=':  return $L != $R;
            case '<':   return $L < $R;
            case '>':   return $L > $R;
            case '<=':  return $L <= $R;
            case '>=':  return $L >= $R;
        }
        PEAR::raiseError('Invalid operator: '. $operator);
    }
    
}
?>
