<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.3 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Auth.php,v 1.3 2005/04/15 13:07:47 mike Exp $
 */

/**
 * Requires Gadget_Service
 */
class_exists('Gadget_Service') or require_once 'Gadget/Service.php';

/**
 * Gadget_Service_Auth
 *
 * Gadget's default authentication service
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @since       1.0
 */
class Gadget_Service_Auth extends Gadget_Service
{
    var $service = SERVICE_AUTH;
    var $authUserId;
    var $authUserName;
    
    /**
     * Execute
     * 
     * @access  protected
     * @return  bool
     * @param   string  $alias
     * @param   string  $password
     * @param   bool    $md5    in|out|false
     */
    function execute($alias = '', $password = '', $md5 = null)
    {
        if (!strlen($alias)) {
            return false;
        }
        
        $user = Gadget::DAO('users');
        $user->alias = $alias;
        $user->active = 1;
        $user->selectSet('id, password');

        $this->authUserName = $alias;
        
        if (!$user->find(true)) {
            return false;
        }
        
        $this->authUserId = $user->id;
        
        switch ($md5)
        {
            // passwords in DB are MD5'ed
            case 'out':
                $realpass = $user->password;
                $comppass = md5($password);
            break;
            
            // passwords coming in are MD5'ed
            case 'in':
                $realpass = md5($user->password);
                $comppass = $password;
            break;
            
            // both passwords are MD5'ed or plain
            default:
                $realpass = $user->password;
                $comppass = $password;
        }
        
        return $comppass === $realpass;
    }
    
    /**
     * On Success
     * 
     * @access  protected
     * @return  true
     */
    function onSuccess()
    {
        parent::onSuccess($this->authUserName .' ('. $this->authUserId .')');
        Gadget::login($this->authUserId);
        return true;
    }
    
    /**
     * On Failure
     * 
     * @access  protected
     * @return  false
     */
    function onFailure()
    {
        parent::onFailure($this->authUserName .' ('. $this->authUserId .')');
        Gadget::redirectLogin();
        return false;
    }
    
}

?>
