<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Template
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.3 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: XmlParser.php,v 1.3 2005/06/07 15:58:03 mike Exp $
 */

/**
 * Requires XML_Parser_Simple
 */
class_exists('XML_Parser_Simple') or require_once 'XML/Parser/Simple.php';

/**
 * Xml document envelope
 * @ignore
 */
if (PHP_VERSION < 5) {
    define('GADGET_TEMPLATE_XMLDOC', '<%s:root>%s</%s:root>');
} else {
    define('GADGET_TEMPLATE_XMLDOC', 
        '<?xml version="1.0" encoding="%s" standalone="yes" ?>'."\n".
        '<%s:root xmlns:%s="http://dev.iworks.at/gadget/">%s</%s:root>'
    );
}

/**#@+
 * Ignore constants
 * @ignore
 */
define('GADGET_TEMPLATE_INGORE_NONE',    0);
define('GADGET_TEMPLATE_INGORE_OPEN',    1);
define('GADGET_TEMPLATE_INGORE_CLOSE',   2);
define('GADGET_TEMPLATE_INGORE_BOTH',    3);
/**#@-*/

/**
 * Gadget_Template_XmlParser
 *
 * XML Parser of Gadget's Template Engine.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Template
 * @since       1.0
 */
class Gadget_Template_XmlParser extends XML_Parser_Simple
{
    /**
     * Case Folding
     * 
     * @var bool
     */
    var $folding = false;
    
    /**
     * Compiler 
     *
     * @var object Gadget_Template_Compiler
     */
    var $Compiler;
    
    /**
     * Namespace 
     *
     * @var string
     */
    var $namespace = 'gadget';
    
    /**
     * Empty Tags 
     *
     * @var array
     */
    var $emptyTags = array('br', 'hr', 'link', 'meta', 'img', 'input', 'param');
    
    /**
     * Form Tags for Gadget Formelement Hooks
     * 
     * @var array
     */
    var $formTags = array('input', 'option');
    
    /**
     * Constructor
     * 
     * @access  public
     * @param   object  Gadget_Template_Compiler $Compiler
     * @param   string  $namespace
     */
    function Gadget_Template_XmlParser(&$Compiler)
    {
        $this->XML_Parser_Simple(GADGET_CHARSET);
        $this->Compiler = &$Compiler;
    }
    
    /**
     * Reset
     * 
     * @access  public
     * @return  mixed
     */
    function reset()
    {
        parent::reset();
        $this->_data[0] = '';
    }
    
    /**
     * Prepare String
     * 
     * @access  protected
     * @return  string
     * @param   string  $xml
     */
    function prepareString($xml)
    {
        /*
         * Base64 encodes:
         *  - processing instructions
         *  - entity declarations, doctypes and comments
         *  - entities
         */
        $preparedXml = preg_replace_callback(
            array('/<\?.*?\?>/s', '/<\!((--)?).*?\1>/s', '/&[[:alnum:]_.-]+;/'),
            array(&$this, 'encodeSpecial'), $xml
        );
        
        if (PHP_VERSION < 5) {
            return sprintf(
                GADGET_TEMPLATE_XMLDOC, 
                $this->Compiler->parseNs, 
                $preparedXml, 
                $this->Compiler->parseNs
            );
        } else {
            return sprintf(
                GADGET_TEMPLATE_XMLDOC, 
                GADGET_CHARSET, 
                $this->Compiler->parseNs, 
                $this->Compiler->parseNs, 
                $preparedXml, 
                $this->Compiler->parseNs
            );
        }
    }
    
    /**
     * Encode Special
     * 
     * @access  protected
     * @return  string
     * @param   array
     */
    function encodeSpecial($matches)
    {
        if (isset($matches[1]) && $this->Compiler->stripComments) {
            return '';
        }
        return '{b64 '. base64_encode($matches[0]) .'}';
    }
    
    /**
     * Parse Template
     * 
     * @access  public
     * @return  string
     * @param   string  $string
     */
    function parseTemplate($string)
    {
        parent::parseString($this->prepareString($string));
        return $this->_data[0];
    }
    
    /**
     * Handle Element
     * 
     * @access  public
     * @return  mixed
     */
    function handleElement($e, $a, $d)
    {
        if ($e === $this->Compiler->parseNs .':root') {
            $data = $d;
        } elseif ($e === $this->Compiler->parseNs .':form') {
            $data = $this->formalize($d);
        } else {
            $data = $this->normalize($e, $a, $d);
        }
        $this->addToData($data);
    }
    
    /**
     * Normalize
     * 
     * @access  public
     * @return  mixed
     */
    function normalize($e, $a, $d)
    {
        // handle form hooks (not already modified by gadget:attributes)
        if (in_array($e, $this->formTags)) {
            if (!count(preg_grep('/^'. $this->Compiler->parseNs .':/', array_keys($a)))) {
                if ($handled = $this->formHandler($e, $a, $d)) {
                    return $handled;
                }
            }
        }
        
        $ignore = GADGET_TEMPLATE_INGORE_NONE;
        $prepend = '';
        $append = '';
        $attributes = '';
        
        foreach ($a as $name => $value) {
            if (!preg_match('/^'.$this->Compiler->parseNs.':(.*)/', $name, $m)) {
                $attributes .= ' '. $name .'="'. $value .'"';
            } else {
                switch ($action = $m[1])
                {
                    case 'attributes':
                        $attributes .= ' '. $value;
                    break;
                    
                    case 'plugin':
                        $d = $this->pluginHandler($value, $d);
                    break;
                    
                    case 'filter':
                        $d = $this->filterHandler($value, $d);
                    break;
                    
                    case 'ignore':
                        switch ($value)
                        {
                            case 'open':
                                $ignore = GADGET_TEMPLATE_INGORE_OPEN;
                            break;
                            case 'close':
                                $ignore = GADGET_TEMPLATE_INGORE_CLOSE;
                            break;
                            case 'both':
                                $ignore = GADGET_TEMPLATE_INGORE_BOTH;
                            break;
                        }
                    break;
                    
                    // call gadget handler
                    default:
                        $this->gadgetHandler($action, $value, $prepend, $append);
                    break;
                }
            }
        }
        
        switch ($ignore)
        {
            case GADGET_TEMPLATE_INGORE_NONE:
                if (in_array($e, $this->emptyTags)) {
                    return $prepend .'<'. $e . $attributes .' />'. $append;
                }
                return $prepend .'<'. $e . $attributes .'>'. $d .'</'. $e .'>'. $append;
            break;
            
            case GADGET_TEMPLATE_INGORE_OPEN:
                return $prepend . $d .'</'. $e .'>'. $append;
            break;
            
            case GADGET_TEMPLATE_INGORE_CLOSE:
                return $prepend .'<'. $e . $attributes .'>'. $d . $append;
            break;
            
            case GADGET_TEMPLATE_INGORE_BOTH:
                return $prepend . $d . $append;
            break;
        }
    }
    
    /**
     * Formalize
     * 
     * @access  public
     * @return  string
     * @param   string  $data
     */
    function formalize($data)
    {
        return  '
	<form {$form->attributes}>
		<div>
			{$form->javascript}
			{$form->hidden}
			'. $data .'
		</div>
	</form>';
    }
    
    /**
     * Filter Handler
     * 
     * @access  protected
     * @return  string
     * @param   string  $filter
     * @param   string  $data
     */
    function filterHandler($filter, $data)
    {
        foreach (explode(',', $filter) as $f) {
            if (function_exists($f)) {
                $data = $f($data);
            }
        }
        return $data;
    }
    
    /**
     * Plugin Handler
     * 
     * @access  protected
     * @return  string
     * @param   string  $plugin
     * @param   string  $data
     */
    function pluginHandler($plugin, $data)
    {
        class_exists('Gadget_Template_Plugin') 
            or require_once 'Gadget/Template/Plugin.php';
        foreach (explode(',', $plugin) as $p) {
            $data = Gadget_Template_Plugin::run($p, $this->Compiler, $data);
        }
        return $data;
    }
    
    /**
     * Gadget Handler 
     *
     * @access  protected
     * @return  void
     * @param   string  $action foreach, while ...
     * @param   string  $value variables etc...
     */
    function gadgetHandler($action, $value, &$prepend, &$append)
    {
        if ($p = $this->Compiler->phpize($action, $value)) {
            $prepend .= $p;
            $append  .= '<?php } ?>';
        }
    }
    
    /**
     * Form Handler
     * 
     * @access  protected
     * @return  bool
     * @param   string  $element
     * @param   array   $attributes
     * @param   string  $data
     */
    function formHandler($element, $attributes, $data)
    {
        switch ($element)
        {
            case 'input':
                if (!isset($attributes['name'])) {
                    return false;
                }
                return $this->buildInputTag($attributes);
            break;
            
            case 'option':
                if (!isset($this->_elStack[$this->_depth - 1])) {
                    return false;
                }
                if ($this->_elStack[$this->_depth - 1]['name'] == 'select') {
                    $sel = $this->_elStack[$this->_depth - 1]['attribs']['name'];
                } elseif (isset($this->_elStack[$this->_depth - 2])) {
                    if ($this->_elStack[$this->_depth - 2]['name'] == 'select') {
                        $sel = $this->_elStack[$this->_depth - 2]['attribs']['name'];
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
                return $this->buildOptionTag($attributes, $data, $sel);
            break;
        }
        return false;
    }
    
    /**
     * Build Input Tag
     * 
     * @access  protected
     * @return  string
     * @param   array   $attr
     */
    function buildInputTag($attr)
    {
        $tag = '<input ';
        $elm = '$elements[\''. $this->Compiler->varize($attr['name'], false) .'\']';
        
        switch ($attr['type'])
        {
            case 'checkbox':
            case 'radio':
                // if element is defined
                $tag .= '<?php if (isset('. $elm .')) { '
                        // checkboc is checked if element evals to true
                     .  'if ('. $elm .') { echo \'checked="checked"\'; } }';
                // if element is not defined and the checked attribute exists
                // mark the checkbox checked
                if (isset($attr['checked'])) {
                    $tag .= ' else { echo \'checked="checked"\'; }';
                    unset($attr['checked']);
                }
                $tag .= ' ?>';
            break;
            
            default:
                // if element is defined
                $tag .= 'value="<?php if (isset('. $elm .')) { '
                        // redefine elements value
                     .  'echo '. $elm .'; }';
                // if element is not defined
                // let predefined value
                if (isset($attr['value'])) {
                    $tag .= ' else { ?>'
                         .  $this->Compiler->varize($attr['value']) .'<?php }';
                    unset($attr['value']);
                }
                $tag .= ' ?>"';
            break;
        }
        
        foreach ($attr as $a => $v) {
            $tag .= ' '. $a .'="'. $v .'"';
        }
        return $tag .' />';
    }
    
    /**
     * Build Option Tag
     * 
     * @access  protected
     * @return  string
     * @param   array   $attr
     */
    function buildOptionTag($attr, $data, $select)
    {
        $tag = '<option ';
        $elm = '$elements[\''. $select .'\']';
        
        $tag .= '<?php if (isset('. $elm .')) { if ('. $elm .'==\''
             .  $this->Compiler->varize($attr['value'], false)
             .  '\') { echo \'selected="selected"\'; } }';
        if (isset($attr['selected'])) {
            $tag .= ' else { echo \'selected="selected"\';}';
            unset($attr['selected']);
        }
        $tag .= ' ?>';
        
        foreach ($attr as $a => $v) {
            $tag .= ' '. $a .'="'. $v .'"';
        }
        return $tag .'>'. $data .'</option>';
    }
}
?>
