<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.2 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Node.php,v 1.2 2005/04/15 13:07:47 mike Exp $
 */

/**
 * Requires Gadget_Tree_Node
 */
class_exists('Gadget_Tree_Node') or require_once 'Gadget/Tree/Node.php';

/**
 * Gadget_Tree_FS_Node
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_FS_Node extends Gadget_Tree_Node 
{
    /**
     * self
     * 
     * Contains the class name - used for addChild().
     * 
     * @access protected 
     * @var string 
     */
    var $self = __CLASS__;

    /**
     * isDir
     * 
     * Whether this filesystem node is a directory.
     * 
     * @access protected 
     * @var bool 
     */
    var $isDir = false;

    /**
     * Directory Separator
     * 
     * @var string
     */
    var $dirSep = DIRECTORY_SEPARATOR;
    
    /**
     * Is Dir
     * 
     * @access public 
     * @return bool Returns true if node is a directory and false if not.
     */
    function isDir()
    {
        return $this->isDir;
    }

    /**
     * Add Data
     * 
     * @access public 
     * @return mixed 
     */
    function addData($data)
    {
        if ($this->isDir) {
            return Gadget_Tree::raiseError(
                'Cannot add data to a directory node.',
                GADGET_TREE_E_INVALID_OP
            );
        }
        class_exists('File') or include_once 'File.php';
        $error = File::write($this->fullPath(), $data, FILE_MODE_APPEND);
        if (Gadget_Tree::isError($error)) {
            return $error;
        }
        return parent::addData($data);
    }

    /**
     * Set Data
     * 
     * @access public 
     * @return mixed 
     */
    function setData($data)
    {
        if ($this->isDir) {
            return Gadget_Tree::raiseError(
                'Cannot set data of a directory node.',
                GADGET_TREE_E_INVALID_OP
            );
        }
        class_exists('File') or include_once 'File.php';
        $error = File::write($this->fullPath(), $data, FILE_MODE_WRITE);
        if (Gadget_Tree::isError($error)) {
            return $error;
        }
        return parent::setData($data);
    }

    /**
     * Set Meta
     * 
     * @access public 
     * @return mixed 
     */
    function setMeta()
    {
        return Gadget_Tree::raiseError(
            'Cannot set meta information of a filesystem node.',
            GADGET_TREE_E_INVALID_OP
        );
    }

    /**
     * Add Meta
     * 
     * @access public 
     * @return mixed 
     */
    function addMeta()
    {
        return Gadget_Tree::raiseError(
            'Cannot add meta information to a filesystem node.',
            GADGET_TREE_E_INVALID_OP
        );
    }

    /**
     * Add New Child
     * 
     * @access public 
     * @return mixed 
     */
    function &addNewChild($name, $isDir = false)
    {
        $child = &new Gadget_Tree_FS_Node($name);
        $child->dirSep = $this->dirSep;
        $child->isDir = $isDir;
        return $this->addChild($child);
    }

    /**
     * Add Child
     * 
     * @access public 
     * @return mixed 
     */
    function &addChild(&$child)
    {
        if (!$this->isDir) {
            return Gadget_Tree::raiseError(
                'Cannot add a child to an filesystem node which is a file.',
                GADGET_TREE_E_INVALID_OP
            );
        }
        if (Gadget_Tree::isError($error = parent::addChild($child))) {
            return $error;
        }
        if (!file_exists($fullPath = $child->fullPath())) {
            if (!($child->isDir ? @mkdir($fullPath) : @touch($fullPath))) {
                return Gadget_Tree::raiseError(
                    $php_errormsg, $child->isDir ? GADGET_TREE_INVALID_PATH :
                        GADGET_TREE_E_INVALID_FILE);
            }
        }
        return $child;
    }

    /**
     * Full Path
     * 
     * Builds the full path of the filesystem node.
     * 
     * @access public 
     * @return string Returns the full path of the node.
     */
    function fullPath()
    {
        return parent::fullPath($this->dirSep);
    }

    /**
     * Set Child
     * 
     * @access public 
     * @return mixed 
     */
    function &setChild(&$child, $pos)
    {
        if (isset($this->childs[$pos])) {
            $this->childs[$pos]->isDir and unlink($this->childs[$pos]->fullPath());
        }
        return parent::setChild($child, $pos);
    }

    /**
     * Unlink
     * 
     * @access public 
     * @return mixed 
     */
    function unlink()
    {
        include_once 'System.php';
        return System::rm(array('-rf', $this->fullPath()));
    }

    /**
     * Get Handle
     * 
     * Returns a file (r+b) or directory handle.
     * 
     * @access public 
     * @return int Returns the resource of a file/directory handle.
     */
    function getHandle()
    {
        static $handle;

        if (!is_resource($handle)) {
            $handle = $this->isDir ?
            opendir($this->fullPath()) :
            fopen($this->fullPath(), 'r+b');
        }
        return $handle;
    }

    /**
     * Read
     * 
     * @access  public
     * @return  mixed
     */
    function read($units = 0)
    {
        return $this->isDir ? $this->readDir($units) : $this->readFile($units);
    }
    
    /**
     * Read Dir
     * 
     * @access public 
     * @return array
     */
    function readDir($count = 0)
    {
        $handle  = $this->getHandle();
        $entries = array();
        while ($count--) {
            $entries[] = readdir($handle);
        }
        return $entries;
    }

    /**
     * Read File
     * 
     * Requires PEAR::File.
     * 
     * @see File::read()
     * @access public 
     * @return mixed 
     * @param int $size Bytes to read from the file. (0 reads all file)
     */
    function readFile($size = 0)
    {
        if ($this->isDir) {
            return Gadget_Tree::raiseError(
                'Cannot call readFile on an directory node.',
                GADGET_TREE_E_INVALID_OP
            );
        }
        class_exists('File') or include_once 'File.php';
        return File::read($this->fullPath(), $size);
    }
}

?>
