<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.2 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Serializer.php,v 1.2 2005/04/15 13:07:47 mike Exp $
 */

/**
 * Requires Gadget_Object
 */
class_exists('Gadget_Object') or require_once 'Gadget/Object.php';

/**
 * Gadget_Tree_Serializer
 * 
 * Sort of abstract base class for custom tree serialization.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_Serializer extends Gadget_Object
{
    /**
     * Serialize Tree
     * 
     * @access  public
     * @return  string
     */
    function serializeTree(&$tree)
    {
        return 
            $this->onStart() .
            $this->serialize($array = array(&$tree->root)) .
            $this->onEnd();
    }
    
    /**
     * Serialize
     * 
     * @access  public
     * @return  string
     * @param   array   $nodes
     */
    function serialize(&$nodes)
    {
        $string = '';
        for ($i = 0, $c = count($nodes); $i < $c; $i++) {
            $string .= $this->onNodeEnter($nodes[$i]);
            if ($nodes[$i]->childs) {
                $string .= $this->onChildsEnter($nodes[$i]);
                $string .= $this->serialize($nodes[$i]->childs);
                $string .= $this->onChildsExit($nodes[$i]);
            }
            $string .= $this->onNodeExit($nodes[$i]);
        }
        return $string;
    }
    
    /**
     * On Start
     * 
     * @access  protected
     * @return  string
     */
    function onStart()
    {
        return '';
    }
    
    /**
     * On End
     * 
     * @access  protected
     * @return  string
     */
    function onEnd()
    {
        return '';
    }
    
    /**
     * On Node Enter
     * 
     * @access  protected
     * @return  string
     */
    function onNodeEnter(&$node)
    {
        return str_repeat(' ', $node->depth()) . $node->name . "\n";
    }
    
    /**
     * On Node Exit
     * 
     * @access  protected
     * @return  string
     */
    function onNodeExit()
    {
        return '';
    }
    
    /**
     * On Childs Enter
     * 
     * @access  protected
     * @return  string
     */
    function onChildsEnter()
    {
        return '';
    }
    
    /**
     * On Childs Exit
     * 
     * @access  protected
     * @return  string
     */
    function onChildsExit()
    {
        return '';
    }
}

?>
