<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.3 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Menu.php,v 1.3 2005/04/15 13:07:47 mike Exp $
 */

class_exists('Gadget_Tree_Serializer') or require_once 'Gadget/Tree/Serializer.php';

/**
 * Gadget_Tree_Serializer_Menu
 * 
 * HTML tree menu serializer
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_Serializer_Menu extends Gadget_Tree_Serializer
{
    /**
     * Menu ID
     * 
     * @var string
     */
    var $menuId = 'TreeMenu';
    
    /**
     * Menu Tag
     * 
     * @var string
     */
    var $menuTag = 'ul';
    
    /**
     * Menu Class
     * 
     * @var string
     */
    var $menuClass = 'Gadget_Tree_Menu';
    
    /**
     * Node Tag
     * 
     * @var string
     */
    var $nodeTag = 'li';
    
    /**
     * Node Class
     * 
     * @var string
     */
    var $nodeClass = '';
    
    /**
     * Column Class for Column 1
     * 
     * @var string
     */
    var $columnClassCol1 = 'Gadget_Tree_Col1';
    
    /**
     * Column Class for Column 2
     * 
     * @var string
     */
    var $columnClassCol2 = 'Gadget_Tree_Col2';

    /**
     * Element IDs to hide while initializing the tree node state
     * 
     * @var array
     */
    var $hideElements = array();
    
    /**
     * Display Style of shown/unhidden nodes/elements
     * 
     * @var string
     */
    var $displayStyle = 'block';
    
    /**
     * Whether to use Cookies for maintaining the State
     * 
     * @var bool
     */
    var $useCookies = true;
    
    /**
     * Whether to use Table Columns (if the actual node name/data is longer)
     * 
     * @var bool
     */
    var $useColumns = true;
    
    /**
     * Whether to use images
     * 
     * @var bool
     */
    var $useImages = true;
    
    /**
     * Whether Nodes that have Childs should be displayed with a Folder Image
     * 
     * @var bool
     */
    var $useFolderImages = true;
    
    /**
     * Expandend Image Suffix
     * 
     * @param   string
     */
    var $expandedImageSuffix = '-expanded';
    
    /**
     * URL Path to Images
     * 
     * @var string
     */
    var $imagePath = '/images';
    
    /**
     * Cookie Path
     * 
     * @var string
     */
    var $cookiePath = '/';
    
    /**
     * Javascript
     * 
     * @var string
     */
    var $javascript = '
    <script type="text/javascript"><!--
        function GTM_changeState(node, postfix, changeExpImage)
        {
            var expanded;
            
            {%hideElements%}
            
            if (node.tagName == "IMG") {
                expanded = GTM_changeImg(node, postfix, changeExpImage);
            }
            
            while (node && node.tagName != "{%nodeTag%}") {
                node = node.parentNode;
            }
            
            if (node && node.tagName == "{%nodeTag%}") {
                var child = node.firstChild;
                while (child && child.tagName != "{%menuTag%}") {
                    child = child.nextSibling;
                }
                if (child && child.tagName == "{%menuTag%}") {
                    child.style.display = (child.style.display == "none") ?
                        "{%displayStyle%}" : "none";
                }
            }
            
            if ({%useCookies%}) {
                GTM_rememberState(node, expanded);
            }
            
            {%unhideElements%}
        }
        
        function GTM_changeImg(node, postfix, changeExpImage)
        {
            var exp_key = "{%expandedImageSuffix%}";
            var off_img = "minus"+ postfix +".gif";
            var on_img = "plus"+ postfix +".gif";
            var path = node.src.substr(0, node.src.lastIndexOf("/"));
            var expanded;
            
            if (node.src == path +"/"+ on_img) {
                node.src = path +"/"+ off_img;
                expanded = true;
            } else {
                node.src = path +"/"+ on_img;
                expanded = false;
            }
            
            if (changeExpImage && node.nextSibling && node.nextSibling.tagName == "IMG") {
                if (expanded) {
                    node.nextSibling.src = node.nextSibling.src.substr(0, node.nextSibling.src.length-4) + exp_key + node.nextSibling.src.substr(node.nextSibling.src.length-4);
                } else {
                    node.nextSibling.src = node.nextSibling.src.replace(exp_key, "");
                }
            }
            
            return expanded;
        }
        
        function GTM_rememberState(node, expanded)
        {
            var key = "Gadget_Tree_Menu_{%menuId%}=";
            var nds = GTM_fetchStateCookie();
            var add = node.getAttribute("ID");
            
            if (expanded) {
                if (!GTM_lookupCookieState(add)) {
                    nds += add +",";
                }
            } else {
                if (GTM_lookupCookieState(add)) {
                    nds = nds.replace(add, "").replace(/,{2,}/, ",");
                }
            }
            
            document.cookie = key + nds + "; path={%cookiePath%}";
        }
        
        function GTM_fetchStateCookie()
        {
            var key = "Gadget_Tree_Menu_{%menuId%}=";
            var pos = document.cookie.indexOf(key);
            var len = document.cookie.length;
            
            if (pos == -1) {
                return "";
            }
            pos += key.length;
            
            return document.cookie.substring(pos, len).replace(/;.*/,"");
        }
        
        function GTM_lookupCookieState(nodeId)
        {
            return -1 != GTM_fetchStateCookie().indexOf(nodeId +",");
        }
        
        function GTM_initState(nodeId, postfix, expimg, hasimg)
        {
            if (!{%useCookies%} || !GTM_lookupCookieState(nodeId)) {
                if ({%useImages%}) {
                    var imgNodes = document.getElementById(nodeId).firstChild.getElementsByTagName("IMG");
                    var imgCount = hasimg + ({%useFolderImages%} ? 2 : 1);
                    GTM_changeState(imgNodes.item(imgNodes.length - imgCount), postfix, expimg);
                } else {
                    GTM_changeState(document.getElementById(nodeId), postfix, expimg);
                }
            }
        }
    //--></script>
';
    
    /**
     * Stylesheet
     * 
     * @var string
     */
    var $stylesheet = '
    <style type="text/css">
        {%menuTag%}#{%menuId%}, 
        {%menuTag%}#{%menuId%} {%nodeTag%}, 
        {%menuTag%}#{%menuId%} {%menuTag%} {
            padding:            0;
            margin:             0;
        }
        {%menuTag%}#{%menuId%} {%nodeTag%} {
            list-style-type:    none;
        }
        {%menuTag%}#{%menuId%} img {
            vertical-align:     middle;
        }
        {%menuTag%}#{%menuId%} table {
            width:              auto;
            border-collapse:    collapse;
        }
        {%menuTag%}#{%menuId%} table td.{%classCol1} {
            white-space:        pre;
        }
    </style>
';

    /**
     * Node IDs to hide initially (computed at run-time)
     * 
     * @var array
     */
    var $nodeIds = array();
    
    /**
     * Node Count
     * 
     * @var int
     */
    var $nodeCnt = 0;
    
    /**
     * On Start
     * 
     * @access  protected
     * @return  string
     */
    function onStart()
    {
        $rj = array(
            '{%displayStyle%}'          => $this->displayStyle,
            '{%nodeTag%}'               => strToUpper($this->nodeTag),
            '{%menuTag%}'               => strToUpper($this->menuTag),
            '{%useCookies%}'            => $this->useCookies ? 'true' : 'false',
            '{%cookiePath%}'            => $this->cookiePath,
            '{%menuId%}'                => $this->menuId,
            '{%useImages%}'             => $this->useImages ? 'true' : 'false',
            '{%useFolderImages%}'       => $this->useFolderImages ? 'true' : 'false',
            '{%expandedImageSuffix%}'   => $this->expandedImageSuffix,
            '{%hideElements%}'          => $this->hiddenElements(true),
            '{%unhideElements%}'        => $this->hiddenElements(false),
        );
        
        $rs = array(
            '{%nodeTag%}'   => $this->nodeTag,
            '{%menuTag%}'   => $this->menuTag,
            '{%menuId%}'    => $this->menuId,
            '{%classCol1}'  => $this->columnClassCol1,
        );
        
        $this->nodeCnt = 0;
        $this->nodeIds = array();
        
        return 
            str_replace(array_keys($rj), $rj, $this->javascript) .
            str_replace(array_keys($rs), $rs, $this->stylesheet) .'<'. 
            $this->menuTag .' id="'. $this->menuId .'" class="'. 
            $this->menuClass .'">';
    }
    
    /**
     * On End
     * 
     * @access  protected
     * @return  string
     */
    function onEnd()
    {
        $string =   '</'. $this->menuTag .">\n"
                .   '<script type="text/javascript"><!-- '."\n"
                .   $this->hiddenElements(true);
        foreach (array_reverse($this->nodeIds) as $id => $n) {
            $string .= "\nGTM_initState('$id', '{$n['postfix']}', {$n['imgexp']}, {$n['hasimg']});";
        }
        return $string ."\n". $this->hiddenElements(false) ."//--></script>\n";
    }
    
    /**
     * On Node Enter
     * 
     * @access  protected
     * @return  string
     */
    function onNodeEnter(&$node)
    {
        $string = '<'. $this->nodeTag .' class="'. $this->nodeClass;
        if ($class = $node->getMeta('class')) {
            $string .= ' '. $class;
        }
        $string .= '"';
        if (!$id = $node->getMeta('id')) {
            $node->addMeta('id', $id = $this->menuId .'_Node_'. $this->nodeCnt++);
        }
        $string .= ' id="'. $id .'"';
        
        $attributes = '';
        foreach ((array) $node->getMeta('attributes') as $attr => $value) {
            $attributes .= ' '. $attr .'="'. $value .'"';
        }
        foreach ((array) $node->getMeta('events') as $event => $action) {
            $attributes .= ' '. $event .'="'. $action .'"';
        }
        
        if ($this->useColumns) {
            $string .= '><table'. $attributes .'><tr><td class="'. $this->columnClassCol1 .'">';
        } else {
            $string .= '><div'. $attributes .'>';
        }
        
        if ($this->useImages) {
            // spanning lines
            $lines = '';
            for ($p = &$node->parent; $p; $p = &$p->parent) {
                $lines = '<img alt="" src="'. $this->imagePath .'/line'. 
                    $this->imagePostfix($p) .'.gif" />'. $lines;
            }
            $string .= $lines;
            
            $postfix = $this->imagePostfix($node);
            
            if ($node->childs) {
                if (!empty($node->expanded)) {
                    for ($p = &$node->parent; $p; $p = &$p->parent) {
                        unset($this->nodeIds[$p->getMeta('id')]);
                    }
                } elseif (!$this->cookieExpandsNode($id)) {
                    // add node to be closed
                    $this->nodeIds[$id] = array(
                        'postfix' => $postfix, 
                        'imgexp'  => (int) $node->getMeta('imgexp'),
                        'hasimg'  => (int) ($node->getMeta('img') || $node->getMeta('imgsrc')),
                    );
                }
                
                $string .= '<img alt="" src="'. $this->imagePath .'/minus'. 
                    $postfix .'.gif" onclick="GTM_changeState(this, \''.
                    $postfix .'\', '. (int) $node->getMeta('imgexp') .
                    ');" />';
                
                if ($this->useFolderImages) {
                    $string .= '<img alt="" src="'. $this->imagePath .
                        '/folder.gif" />';
                }
            } else {
                $string .= '<img alt="" src="'. $this->imagePath .'/branch'. 
                    $postfix .'.gif" />';
            }
            
            if ($img = $node->getMeta('img')) {
                $string .= $img;
            } elseif ($img = $node->getMeta('imgsrc')) {
                $string .= '<img alt="'. $node->getMeta('imgalt') .'" src="'. 
                    $this->imagePath .'/'. $img .'" />';
            }
        }
        
        if ($this->useColumns) {
            $string .= '</td><td class="'. $this->columnClassCol2 .'">';
        }
        
        if ($href = $node->getMeta('href')) {
            $string .= '<a href="'. $href .'"';
            if ($hrefclass = $node->getMeta('hrefclass')) {
                $string .= ' class="'. $hrefclass .'"';
            }
            $string .= '>'. $node->name .'</a>';
        } elseif (strlen($node->data)) {
            $string .= ' ';
        } else {
            $string .= $node->name;
        }
        
        return $string . $node->data . ($this->useColumns ? '</td></tr></table>':'</div>');
    }
    
    /**
     * On Node Exit
     * 
     * @access  protected
     * @return  string
     */
    function onNodeExit()
    {
        return '</'. $this->nodeTag .'>';
    }
    
    /**
     * On Node Exit
     * 
     * @access  protected
     * @return  string
     */
    function onChildsEnter(&$node)
    {
        return '<'. $this->menuTag .' class="'. $this->menuClass .'">';
    }

    /**
     * On Node Exit
     * 
     * @access  protected
     * @return  string
     */
    function onChildsExit()
    {
        return '</'. $this->menuTag .'>';
    }
    
    /**
     * Image Postfix
     * 
     * @access  protected
     * @return  string
     * @param   object  Gadget_Tree_Menu_Node $node
     */
    function imagePostfix(&$node)
    {
        if (!$node->following) {
            return 'bottom';
        } elseif (!$node->preceding && !$node->parent) {
            return 'top';
        } else {
            return '';
        }
    }
    
    /**
     * Cookie expands Node
     * 
     * @access  protected
     * @return  bool
     * @param   string  $id
     */
    function cookieExpandsNode($id)
    {
        if (!$this->useCookies || !isset($_COOKIE['Gadget_Tree_Menu_'. $this->menuId])) {
            return false;
        }
        return false !== strstr($_COOKIE['Gadget_Tree_Menu_'. $this->menuId], $id .',');
    }
    
    /**
     * Hidden Elements
     * 
     * @access  protected
     * @return  string
     * @param   bool    $hidden
     */
    function hiddenElements($hidden)
    {
        $string = '';
        foreach ($this->hideElements as $id) {
            $string .=
                'try { document.getElementById("'. $id .'").style.display = "'. 
                ($hidden ? 'none' : $this->displayStyle) . 
                '"; } catch (__EX__) {}'."\n";
        }
        return $string;
    }
    
}

?>
