<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.3 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: XML.php,v 1.3 2005/04/21 11:06:13 mike Exp $
 */

/**
 * Requires Gadget_Tree
 */
class_exists('Gadget_Tree') or require_once 'Gadget/Tree.php';

/**
 * Requires Gadget_Tree_XML_Node
 */
class_exists('Gadget_Tree_XML_Node') or require_once 'Gadget/Tree/XML/Node.php';

/**
 * Requires Gadget_Tree_XML_Parser
 */


/**
 * Gadget_Tree_XML
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_XML extends Gadget_Tree 
{
    /**
     * Encoding
     * 
     * @var string
     */
    var $encoding = 'ISO-8859-1';
    
    /**
     * Create
     * 
     * See Gadget_Tree::create().
     * 
     * @access  public 
     * @return  object 
     * @param   mixed   $params 
     * @param   string  $class 
     */
    function &create($params, $class = __CLASS__)
    {
        return parent::create($params, $class);
    }

    /**
     * From File
     * 
     * Build the tree from the content of an XML file.
     * 
     * @access  public 
     * @return  object  Gadget_Tree_XML_Node The root node of the tree.
     * @param   string  $file The path to the XML file.
     */
    function &fromFile($file, $encoding = null)
    {
        class_exists('Gadget_Tree_XML_Parser') or 
            require_once 'Gadget/Tree/XML/Parser.php';
        $parser = &new Gadget_Tree_XML_Parser($encoding);

        if (Gadget_Tree::isError($error = $parser->setInputFile($file))) {
            return $error;
        }
        if (Gadget_Tree::isError($error = $parser->parse())) {
            return $error;
        }

        return $this->root = &$parser->getRoot();
    }

    /**
     * From String
     * 
     * Build the tree from an XML string.
     * 
     * @access  public 
     * @return  mixed   Returns the root node on success or false.
     * @param   string  $xml XML string
     */
    function &fromString($xml, $encoding = null)
    {
        class_exists('Gadget_Tree_XML_Parser') or 
            require_once 'Gadget/Tree/XML/Parser.php';
        $parser = &new Gadget_Tree_XML_Parser($encoding);
        if (Gadget_Tree::isError($error = $parser->parseString($xml))) {
            return $error;
        }
        return $this->root = &$parser->getRoot();
    }

    /**
     * From Default
     * 
     * Tries to guess the type of the source.  Calls Gadget_Tree_XML::fromFile(),
     * Gadget_Tree_XML::fromString() and Gadget_Tree::fromNode in this order.
     * 
     * @access  public 
     * @return  mixed   Returns the root node on success or false.
     */
    function &fromDefault(&$from)
    {
        return parent::from($from, array('File' => true, 'String' => true));
    }

    /**
     * To File
     * 
     * Overrides Gadget_Tree::toFile().  Prepends the XML declaration with proper
     * encoding information to Gadget_Tree_XML::toString() and saves to a file.
     * 
     * @access  public 
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   string  $file The path of the file to save to.
     * @param   int     $indent The count of indenting space characters to use.
     * @param   array   $options Serializer options
     */
    function toFile($file, $options = array())
    {
        return parent::fputs($file, $this->toString($options));
    }

    /**
     * To String
     * 
     * Overrides Gadget_Tree::toString() to provide the options for a different
     * output encoding using iconv().
     * 
     * @see Gadget_Tree_XML::encoding, Gadget_Tree_XML::setOptions()
     * @access  public 
     * @return  string 
     * @param   sarray  $options Serializer options
     */
    function toString($options = array())
    {
        $string = Gadget_Tree_XML::serialize($this, $options);
        if (isset($options['encoding']) && 
                strcasecmp($options['encoding'], $this->encoding)) {
            $string = iconv($this->encoding, $options['encoding'], $string);
        }
        return $string;
    }

    /**
     * Serialize
     * 
     * @static
     * @access  public 
     * @return  string  XML
     * @param   object  Gadget_Tree_XML $tree
     * @param   array   $options Serializer options
     * @param   string  $serilaizer Gadget_Tree_Serializer class
     */
    function serialize(&$tree, $options = array(), $serializer = 'Gadget_Tree_Serializer_XML')
    {
        return parent::serialize($tree, $options, $serializer);
    }
}

?>
